==> java-ui-241.12166-EAP-CANDIDATE-SNAPSHOT-sources.jar: com/intellij/jarRepository/repositoryLibrarySynchronizers.kt <==
// Copyright 2000-2023 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package com.intellij.jarRepository

import com.intellij.openapi.Disposable
import com.intellij.openapi.project.Project
import com.intellij.openapi.roots.RootProvider
import com.intellij.openapi.roots.impl.libraries.LibraryEx
import com.intellij.openapi.roots.libraries.Library
import com.intellij.openapi.roots.libraries.LibraryTable
import com.intellij.openapi.roots.libraries.LibraryTablesRegistrar
import com.intellij.platform.backend.workspace.WorkspaceModelChangeListener
import com.intellij.platform.workspace.jps.entities.*
import com.intellij.platform.workspace.storage.EntityChange
import com.intellij.platform.workspace.storage.EntityStorage
import com.intellij.platform.workspace.storage.VersionedStorageChange
import com.intellij.workspaceModel.ide.impl.legacyBridge.library.ProjectLibraryTableBridgeImpl.Companion.libraryMap

internal class GlobalChangedRepositoryLibrarySynchronizer(private val queue: LibrarySynchronizationQueue,
                                                          private val disposable: Disposable)
  : LibraryTable.Listener, RootProvider.RootSetChangedListener {
  override fun beforeLibraryRemoved(library: Library) {
    if (library is LibraryEx) {
      library.rootProvider.removeRootSetChangedListener(this)
      queue.revokeSynchronization(library)
    }
  }

  override fun afterLibraryAdded(library: Library) {
    if (library is LibraryEx) {
      library.rootProvider.addRootSetChangedListener(this, disposable)
      queue.requestSynchronization(library)
    }
  }

  override fun rootSetChanged(wrapper: RootProvider) {
    for (library in LibraryTablesRegistrar.getInstance().libraryTable.libraries) {
      if (library.rootProvider == wrapper) {
        if (library is LibraryEx) {
          queue.requestSynchronization(library)
        }
        break
      }
    }
  }

  fun installOnExistingLibraries() {
    getGlobalAndCustomLibraryTables()
      .flatMap { it.libraries.asIterable() }
      .filterIsInstance<LibraryEx>()
      .forEach { it.rootProvider.addRootSetChangedListener(this, disposable) }
  }
}

internal fun getGlobalAndCustomLibraryTables(): Sequence<LibraryTable> {
  return LibraryTablesRegistrar.getInstance().customLibraryTables.asSequence() + LibraryTablesRegistrar.getInstance().libraryTable
}

internal class ChangedRepositoryLibrarySynchronizer(private val project: Project,
                                                    private val queue: LibrarySynchronizationQueue) : WorkspaceModelChangeListener {
  /**
   * This is a flag indicating that the [beforeChanged] method was called.
   * Since we subscribe using the code, this may lead to IDEA-324532.
   * With this flag, we skip the "after" event if the before event wasn't called.
   */
  private var beforeCalled = false

  override fun beforeChanged(event: VersionedStorageChange) {
    beforeCalled = true
    for (change in event.getChanges(LibraryEntity::class.java)) {
      val removed = change as? EntityChange.Removed ?: continue
      val library = findLibrary(removed.entity.symbolicId, event.storageBefore)
      if (library != null) {
        queue.revokeSynchronization(library)
      }
    }

    for (change in event.getChanges(ModuleEntity::class.java)) {
      val (oldLibDeps, newLibDeps) = when (change) {
        is EntityChange.Added -> continue
        is EntityChange.Removed -> change.entity.libraryDependencies() to emptySet()
        is EntityChange.Replaced -> change.oldEntity.libraryDependencies() to change.newEntity.libraryDependencies()
      }

      oldLibDeps
        .filterNot { newLibDeps.contains(it) }
        .mapNotNull { findLibrary(it, event.storageBefore) }
        .forEach { queue.revokeSynchronization(it) }
    }
  }

  override fun changed(event: VersionedStorageChange) {
    if (!beforeCalled) return
    beforeCalled = false
    var libraryReloadRequested = false

    for (change in event.getChanges(LibraryEntity::class.java)) {
      val entity = when (change) {
                     is EntityChange.Added -> change.entity
                     is EntityChange.Replaced -> change.newEntity
                     is EntityChange.Removed -> null
                   } ?: continue
      val library = findLibrary(entity.symbolicId, event.storageAfter)
      if (library != null) {
        queue.requestSynchronization(library)
        libraryReloadRequested = true
      }
    }


    for (change in event.getChanges(ModuleEntity::class.java)) {
      val (oldLibDeps, newLibDeps) = when (change) {
        is EntityChange.Removed -> continue
        is EntityChange.Added -> emptySet<LibraryDependency>() to change.entity.libraryDependencies()
        is EntityChange.Replaced -> change.oldEntity.libraryDependencies() to change.newEntity.libraryDependencies()
      }

      newLibDeps.filterNot { oldLibDeps.contains(it) }.mapNotNull { findLibrary(it, event.storageAfter) }.forEach {
        queue.requestSynchronization(it)
        libraryReloadRequested = true
      }
    }

    if (libraryReloadRequested) {
      queue.flush()
    }
  }

  private fun findLibrary(libraryId: LibraryId, storage: EntityStorage): LibraryEx? {
    val library = when (val tableId = libraryId.tableId) {
      is LibraryTableId.GlobalLibraryTableId ->
        LibraryTablesRegistrar.getInstance().getLibraryTableByLevel(tableId.level, project)?.getLibraryByName(libraryId.name)
      else -> libraryId.resolve(storage)?.let { storage.libraryMap.getDataByEntity(it) }
    }
    return library as? LibraryEx
  }

  private fun findLibrary(libDep: LibraryDependency, storage: EntityStorage): LibraryEx? =
    findLibrary(libDep.library, storage)

  private fun ModuleEntity.libraryDependencies(): Set<LibraryDependency> =
    dependencies.filterIsInstance<LibraryDependency>().toHashSet()
}

==> java-ui-241.12662-EAP-CANDIDATE-SNAPSHOT-sources.jar: com/intellij/jarRepository/repositoryLibrarySynchronizers.kt <==
// Copyright 2000-2023 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package com.intellij.jarRepository

import com.intellij.openapi.Disposable
import com.intellij.openapi.project.Project
import com.intellij.openapi.roots.RootProvider
import com.intellij.openapi.roots.impl.libraries.LibraryEx
import com.intellij.openapi.roots.libraries.Library
import com.intellij.openapi.roots.libraries.LibraryTable
import com.intellij.openapi.roots.libraries.LibraryTablesRegistrar
import com.intellij.platform.backend.workspace.WorkspaceModelChangeListener
import com.intellij.platform.workspace.jps.entities.*
import com.intellij.platform.workspace.storage.EntityChange
import com.intellij.platform.workspace.storage.EntityStorage
import com.intellij.platform.workspace.storage.VersionedStorageChange
import com.intellij.workspaceModel.ide.impl.legacyBridge.library.ProjectLibraryTableBridgeImpl.Companion.libraryMap

internal class GlobalChangedRepositoryLibrarySynchronizer(private val queue: LibrarySynchronizationQueue,
                                                          private val disposable: Disposable)
  : LibraryTable.Listener, RootProvider.RootSetChangedListener {
  override fun beforeLibraryRemoved(library: Library) {
    if (library is LibraryEx) {
      library.rootProvider.removeRootSetChangedListener(this)
      queue.revokeSynchronization(library)
    }
  }

  override fun afterLibraryAdded(library: Library) {
    if (library is LibraryEx) {
      library.rootProvider.addRootSetChangedListener(this, disposable)
      queue.requestSynchronization(library)
    }
  }

  override fun rootSetChanged(wrapper: RootProvider) {
    for (library in LibraryTablesRegistrar.getInstance().libraryTable.libraries) {
      if (library.rootProvider == wrapper) {
        if (library is LibraryEx) {
          queue.requestSynchronization(library)
        }
        break
      }
    }
  }

  fun installOnExistingLibraries() {
    getGlobalAndCustomLibraryTables()
      .flatMap { it.libraries.asIterable() }
      .filterIsInstance<LibraryEx>()
      .forEach { it.rootProvider.addRootSetChangedListener(this, disposable) }
  }
}

internal fun getGlobalAndCustomLibraryTables(): Sequence<LibraryTable> {
  return LibraryTablesRegistrar.getInstance().customLibraryTables.asSequence() + LibraryTablesRegistrar.getInstance().libraryTable
}

internal class ChangedRepositoryLibrarySynchronizer(private val project: Project,
                                                    private val queue: LibrarySynchronizationQueue) : WorkspaceModelChangeListener {
  /**
   * This is a flag indicating that the [beforeChanged] method was called.
   * Since we subscribe using the code, this may lead to IDEA-324532.
   * With this flag, we skip the "after" event if the before event wasn't called.
   */
  private var beforeCalled = false

  override fun beforeChanged(event: VersionedStorageChange) {
    beforeCalled = true
    for (change in event.getChanges(LibraryEntity::class.java)) {
      val removed = change as? EntityChange.Removed ?: continue
      val library = findLibrary(removed.entity.symbolicId, event.storageBefore)
      if (library != null) {
        queue.revokeSynchronization(library)
      }
    }

    for (change in event.getChanges(ModuleEntity::class.java)) {
      val (oldLibDeps, newLibDeps) = when (change) {
        is EntityChange.Added -> continue
        is EntityChange.Removed -> change.entity.libraryDependenciesAsSequence() to emptySet()
        is EntityChange.Replaced -> change.oldEntity.libraryDependenciesAsSequence() to change.newEntity.libraryDependencies()
      }

      oldLibDeps.filterNot { newLibDeps.contains(it) }
        .mapNotNull { findLibrary(it, event.storageBefore) }
        .forEach { queue.revokeSynchronization(it) }
    }
  }

  override fun changed(event: VersionedStorageChange) {
    if (!beforeCalled) return
    beforeCalled = false
    var libraryReloadRequested = false

    for (change in event.getChanges(LibraryEntity::class.java)) {
      val entity = when (change) {
                     is EntityChange.Added -> change.entity
                     is EntityChange.Replaced -> change.newEntity
                     is EntityChange.Removed -> null
                   } ?: continue
      val library = findLibrary(entity.symbolicId, event.storageAfter)
      if (library != null) {
        queue.requestSynchronization(library)
        libraryReloadRequested = true
      }
    }


    for (change in event.getChanges(ModuleEntity::class.java)) {
      val (oldLibDeps, newLibDeps) = when (change) {
        is EntityChange.Removed -> continue
        is EntityChange.Added -> emptySet<LibraryDependency>() to change.entity.libraryDependenciesAsSequence()
        is EntityChange.Replaced -> change.oldEntity.libraryDependencies() to change.newEntity.libraryDependenciesAsSequence()
      }

      newLibDeps.filterNot { oldLibDeps.contains(it) }
        .mapNotNull { findLibrary(it, event.storageAfter) }
        .forEach {
          queue.requestSynchronization(it)
          libraryReloadRequested = true
        }
    }

    if (libraryReloadRequested) {
      queue.flush()
    }
  }

  private fun findLibrary(libraryId: LibraryId, storage: EntityStorage): LibraryEx? {
    val library = when (val tableId = libraryId.tableId) {
      is LibraryTableId.GlobalLibraryTableId ->
        LibraryTablesRegistrar.getInstance().getLibraryTableByLevel(tableId.level, project)?.getLibraryByName(libraryId.name)
      else -> libraryId.resolve(storage)?.let { storage.libraryMap.getDataByEntity(it) }
    }
    return library as? LibraryEx
  }

  private fun findLibrary(libDep: LibraryDependency, storage: EntityStorage): LibraryEx? =
    findLibrary(libDep.library, storage)

  private fun ModuleEntity.libraryDependenciesAsSequence(): Sequence<LibraryDependency> =
    dependencies.asSequence().filterIsInstance<LibraryDependency>()

  private fun ModuleEntity.libraryDependencies(): Set<LibraryDependency> =
    libraryDependenciesAsSequence().toHashSet()
}
